exportPattern("^[[:alpha:]]+")
importFrom(ash, ash1)
importFrom(Hmisc, wtd.quantile)
S3method(as.Message, histogram)
S3method(as.histogram, Message)
importFrom("graphics", "abline", "arrows", "axis", "plot", "rect")
importFrom("stats", "aggregate", "approxfun", "knots")
importFrom("utils", "head", "tail")
importFrom("stats", "weighted.mean")
